from InquirerPy import inquirer


def prompt_decorator(func):
    def wrapper(*args, **kwargs):
        if kwargs.get("multiselect"):
            keybindings = {
                "toggle": [
                    {"key": "c-z"}
                ],
                "toggle-all-true": [
                    {"key": "c-a"}
                ],
                "toggle-all-false": [
                    {"key": "c-a"}
                ],
            }
        else:
            keybindings = {
                "answer": [
                    {"key": "enter"},
                    {"key": "c-z"}
                ],
            }

        if kwargs.get("long_instruction"):
            long_instruction = f"{kwargs['long_instruction']}\nReturn to the main menu at any time with Control + C."
        else:
            long_instruction = "Return to the main menu at any time with Control + C."

        kwargs["keybindings"] = keybindings
        kwargs["long_instruction"] = long_instruction
        kwargs["qmark"] = "•"
        kwargs["amark"] = "✓"
        kwargs["mandatory_message"] = "You can't skip this."
        kwargs["raise_keyboard_interrupt"] = True

        return func(*args, **kwargs)

    return wrapper


checkbox = prompt_decorator(inquirer.checkbox)
confirm = prompt_decorator(inquirer.confirm)
expand = prompt_decorator(inquirer.expand)
filepath = prompt_decorator(inquirer.filepath)
fuzzy = prompt_decorator(inquirer.fuzzy)
text = prompt_decorator(inquirer.text)
select = prompt_decorator(inquirer.select)
number = prompt_decorator(inquirer.number)
rawlist = prompt_decorator(inquirer.rawlist)
secret = prompt_decorator(inquirer.secret)



import json
import plistlib
import re
import subprocess
import sys
import zipfile
from datetime import datetime

import click
import inflect as ifl
from InquirerPy.base import Choice
from colour import Color
from path import Path
from prompt_toolkit.document import Document
from prompt_toolkit.validation import Validator, ValidationError

import helpers
import importers
import prompts


inflect = ifl.engine()


def add_schemes():
    source = prompts.select(
        message="Where from?",
        choices=[
            Choice(name="Built-in color schemes", value="builtin"),
            Choice(name="GitHub repository", value="github"),
            Choice(name="Local files", value="local"),
        ]
    ).execute()

    if source == "builtin":
        importers.add_from_builtins()
    elif source == "github":
        importers.add_from_github()
    elif source == "local":
        importers.add_from_files()


def remove_schemes():
    scheme_settings = helpers.get_scheme_settings()

    scheme_choices = [name for name in scheme_settings.keys()]

    if not scheme_choices:
        click.echo(helpers.make_error_text("You haven't added any color schemes."))
        raise KeyboardInterrupt

    if len(scheme_choices) == 1:
        schemes = [scheme_choices[0]]
        click.echo(helpers.make_info_text("You only have one removable color scheme."))
    else:
        scheme_choices.sort(key=lambda c: c.lower())

        schemes = prompts.fuzzy(
            message="Choose color schemes to remove.",
            instruction="Type to search.",
            long_instruction=helpers.MULTISELECT_CONTROLS,
            choices=scheme_choices,
            multiselect=True,
            transformer=lambda result: f"Removing {inflect.no('color scheme', len(result))}.",
        ).execute()

    if len(schemes) <= 5:
        scheme_indicator = inflect.join([scheme for scheme in schemes])
    else:
        scheme_indicator = inflect.no('color scheme', len(schemes))

    if prompts.confirm(f"Remove {scheme_indicator}?").execute():
        for scheme in schemes:
            del scheme_settings[scheme]

        helpers.save_scheme_settings(scheme_settings)

        click.echo(helpers.make_success_text(f"Removed {scheme_indicator}. Restart iTerm2 to apply your changes."))


def export_schemes():
    scheme_settings = helpers.get_scheme_settings()

    scheme_choices: list[str] = [name for name in scheme_settings.keys()]

    if not scheme_choices:
        click.echo(helpers.make_error_text("You haven't added any color schemes."))
        raise KeyboardInterrupt

    if len(scheme_choices) == 1:
        schemes = [scheme_choices[0]]
        click.echo(helpers.make_info_text(f"You only have one exportable color scheme. ({scheme_choices[0]})"))
    else:
        scheme_choices.sort(key=lambda c: c.lower())

        schemes: list[str] = prompts.fuzzy(
            message="Choose color schemes to export.",
            instruction="Type to search.",
            long_instruction=helpers.MULTISELECT_CONTROLS,
            choices=scheme_choices,
            multiselect=True,
            transformer=lambda result: f"Exporting {inflect.no('color scheme', len(result))}.",
        ).execute()

    format_choices = [
        Choice(name="iTermColors", value="itermcolors"),
        Choice(name="JSON", value="json"),
        Choice(name="XML", value="xml"),
        Choice(name="Property list", value="plist"),
    ]

    export_format: str = prompts.select(
        message="Choose an export format.",
        instruction="Only iTermColors files can be imported back into iTerm2.",
        choices=format_choices,
    ).execute()

    export_dir: Path = prompts.filepath(
        message="Choose a directory to export to.",
        instruction="Relative paths and symbolic links are fully supported.",
        long_instruction=f"The current working directory is {Path.getcwd()}.",
        validate=lambda result: Path(result).expand().isdir(),
        invalid_message="That's not a directory.",
        filter=lambda result: Path(result).expand().realpath(),
        transformer=lambda result: Path(result).expand().realpath(),
    ).execute()

    if len(schemes) <= 5:
        scheme_indicator = inflect.join([scheme for scheme in schemes])
    else:
        scheme_indicator = inflect.no("color scheme", len(schemes))

    date = datetime.today().strftime('%x').replace('/', '-')
    if len(schemes) == 1:
        scheme = schemes[0]
        export_path = export_dir / f"{scheme}.{export_format}"

        if export_path.exists():
            if not prompts.confirm(f"{export_path} already exists. Overwrite it?").execute():
                raise KeyboardInterrupt

        if export_format == "json":
            json.dump(scheme_settings[scheme], export_path.open("w"))
        else:
            plist_format = plistlib.FMT_BINARY if export_format == "plist" else plistlib.FMT_XML
            plistlib.dump(scheme_settings[scheme], export_path.open("wb"), fmt=plist_format)
    else:
        export_path = export_dir / f"schemarin_{export_format}_export_{date}.zip"

        if export_path.exists():
            if not prompts.confirm(f"{export_path} already exists. Overwrite it?").execute():
                raise KeyboardInterrupt

        with zipfile.ZipFile(export_dir / f"schemarin_{export_format}_export_{date}.zip", mode="w") as archive:
            for scheme in schemes:
                export_name = f"{scheme}.{export_format}"
                if export_format == "json":
                    archive.writestr(export_name, json.dumps(scheme_settings[scheme]))
                else:
                    plist_format = plistlib.FMT_BINARY if export_format == "plist" else plistlib.FMT_XML
                    archive.writestr(export_name, plistlib.dumps(scheme_settings[scheme], fmt=plist_format))

    click.echo(helpers.make_success_text(f"Exported {scheme_indicator} to {export_path}."))


def preview_schemes():
    class PreviewValidator(Validator):
        def validate(self, document: Document) -> None:
            preview_scheme = scheme_settings[document.text]
            for k, v in preview_scheme.items():
                click.echo(get_ansi_code(k, v), nl=False)

            raise ValidationError(message=f"Previewing {document.text}.")

    def get_ansi_code(attr_name: str, attr_data: dict) -> str:
        if re.match(r"Ansi \d+ Color", attr_name):
            ansi_id = int(re.search(r"\d+", attr_name).group())
            attr_code = ["black", "red", "green", "yellow", "blue", "magenta", "cyan", "white",
                         "br_black", "br_red", "br_green", "br_yellow", "br_blue", "br_magenta",
                         "br_cyan", "br_white"][ansi_id]
        else:
            attr_code = {
                "Foreground Color": "fg",
                "Background Color": "bg",
                "Bold Color": "bold",
                "Selection Color": "selbg",
                "Selected Text Color": "selfg",
                "Cursor Color": "curbg",
                "Cursor Text Color": "curfg",
            }.get(attr_name, "")

        red = attr_data["Red Component"]
        green = attr_data["Green Component"]
        blue = attr_data["Blue Component"]
        color_hex = Color(rgb=(red, green, blue)).hex.lstrip("#")

        return f"\033]1337;SetColors={attr_code}={color_hex}\033\\"

    if not Path("~/.iterm2/it2check").expand().exists():
        msg = "Please install iTerm2's shell integration."
        info_msg = "Learn more: https://iterm2.com/documentation-shell-integration.html"
        click.echo(helpers.make_error_text(msg))
        click.echo(helpers.make_info_text(info_msg))
        raise KeyboardInterrupt

    if not subprocess.run("~/.iterm2/it2check", stdout=sys.stdout, shell=True).returncode == 0:
        msg = "Please quit Schemarin and relaunch it from within iTerm2."
        click.echo(helpers.make_error_text(msg))
        raise KeyboardInterrupt

    scheme_settings = helpers.get_scheme_settings()

    scheme_choices: list[str] = [name for name in scheme_settings.keys()]

    if not scheme_choices:
        click.echo(helpers.make_error_text("You haven't added any color schemes."))
        raise KeyboardInterrupt

    scheme_choices.sort(key=lambda c: c.lower())

    try:
        prompts.fuzzy(
            message="Choose a color scheme to preview.",
            instruction="Type to search.",
            long_instruction="Preview colors may not be perfectly accurate.",
            choices=scheme_choices,
            validate=PreviewValidator(),
        ).execute()
    except KeyboardInterrupt:
        msg = "To return to your original color scheme, open a new terminal tab or window."
        click.echo(helpers.make_info_text(msg))
        raise KeyboardInterrupt


def main():
    print("Schemarin © 2022-present celsius narhwal. Licensed under MIT (see --license).\n")

    while True:
        try:
            prompts.select(
                message="What would you like to do?",
                choices=[Choice(name="Add color schemes", value=add_schemes),
                         Choice(name="Remove color schemes", value=remove_schemes),
                         Choice(name="Export color schemes", value=export_schemes),
                         Choice(name="Preview color schemes", value=preview_schemes),
                         Choice(name="Quit Schemarin", value=exit)],
                transformer=lambda result: "Goodbye!" if "quit" in result.casefold() else result
            ).execute()()
        except KeyboardInterrupt:
            # Aside from effectively binding Control + C to the main menu, this allows Schemarin to easily
            # force returns to the main menu by explicitly raising KeyboardInterrupt exceptions.
            click.echo(helpers.make_info_text("Returning to the main menu."))
            continue


@click.command()
@click.option("--license", "show_license", is_flag=True, help="See schemarin's license.")
def cli(show_license):
    if not helpers.preflight():
        exit(1)
    elif show_license:
        click.echo((Path(__file__).parent / "LICENSE.md").read_text())
    else:
        main()


if __name__ == "__main__":
    cli()



import platform
import plistlib
import sys
import time

import click
import inflect as ifl
import pyperclip
import toml
from InquirerPy.base import Choice
from github import Github as GitHub
from github.GithubException import BadCredentialsException
from halo import Halo
from loctocat import GitHubAuthenticator
from loctocat import Handler
from loctocat import ServerError
from path import Path
from prompt_toolkit.document import Document
from prompt_toolkit.validation import Validator, ValidationError

import prompts

inflect = ifl.engine()

IT2_PREFS_PATH = Path("~/Library/Preferences/com.googlecode.iterm2.plist").expand()

MULTISELECT_CONTROLS = ("↑/↓: Move up and down\n"
                        "Control + Z: Toggle selection\n"
                        "Control + A: Toggle all on/off\n"
                        "Enter: Confirm\n")


def preflight():
    passing = True

    if sys.platform != "darwin":
        click.echo(make_error_text("Schemarin only works on macOS. Sorry!"))
        passing = False
    elif not IT2_PREFS_PATH.exists():
        click.echo(make_error_text("schemarin couldn't find iTerm2's preferences file. "
                                   "Install iTerm2 from https://iterm2.com. If iTerm2 is installed and you're still "
                                   "seeing this message, file an issue on GitHub: "
                                   "https://github.com/celsiusnarhwal/schemarin/issues"))
        passing = False

    return passing


def get_scheme_indicator(schemes: list) -> str:
    if len(schemes) <= 5:
        return inflect.join([scheme.stem for scheme in schemes])
    else:
        return inflect.no("color scheme", len(schemes))


def get_it2_prefs():
    return plistlib.load(IT2_PREFS_PATH.open("rb"))


def get_scheme_settings():
    return get_it2_prefs()["Custom Color Presets"]


def save_scheme_settings(settings: dict):
    prefs = get_it2_prefs()
    prefs["Custom Color Presets"] = settings
    plistlib.dump(prefs, IT2_PREFS_PATH.open("wb"))


def get_builtin_schemes_dir():
    return Path(__file__).parent / "schemes" / "schemes"


def make_info_text(text: str):
    return f"{click.style('ℹ', fg='blue')} {text}"


def make_success_text(text: str):
    return f"{click.style('✔', fg='green')} {text}"


def make_error_text(text: str):
    return f"{click.style('✖', fg='red')} {text}"


def sign_in_to_github():
    token = None

    class AuthMenuValidator(Validator):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.last_poll_time = None

        def validate(self, document: Document) -> None:
            if document.text == "done":
                if self.last_poll_time and time.time() - self.last_poll_time < 5:
                    raise ValidationError(message="Slow your roll. Try again in a few seconds.")
                else:
                    nonlocal token
                    try:
                        token = authenticator.poll()
                    except ServerError:
                        token = False
                    else:
                        if not token:
                            self.last_poll_time = time.time()
                            raise ValidationError(message="No you didn't.")
            elif document.text == "copy":
                pyperclip.copy(auth_info.user_code)
                raise ValidationError(message="Code copied to clipboard.")
            elif document.text == "open":
                click.launch(auth_info.verification_uri)
                raise ValidationError(message="Browser opened.")

    authenticator = GitHubAuthenticator(client_id="05cf23375abbfca11121", scopes=["repo"])
    authenticator.attach_handler(Handler("authorization_pending", lambda ctx: None, continue_on_error=False))

    auth_info = authenticator.ping()

    auth_menu_choices = [
        Choice(name="I did it", value="done"),
        Choice(name="Copy code to clipboard", value="copy"),
        Choice(name="Open URL in browser", value="open"),
        Choice(name="I'd rather log in with a token", value="token"),
        Choice(name="Go back", value="redo"),
    ]

    auth_menu_result = prompts.select(
        message=f"Go to {auth_info.verification_uri}, enter the code {auth_info.user_code}, and click "
                f"\"Authorize\".",
        choices=auth_menu_choices,
        validate=AuthMenuValidator(),
        transformer=lambda result: f"Signed in as {GitHub(token).get_user().login}" if token else result
    ).execute()

    if auth_menu_result == "token":
        return sign_in_to_github_with_token()
    elif auth_menu_result == "redo":
        return None
    elif auth_menu_result == "done":
        return token


def sign_in_to_github_with_token():
    class TokenValidator(Validator):
        def validate(self, document: Document) -> None:
            try:
                # noinspection PyStatementEffect
                GitHub(document.text).get_user().login
            except BadCredentialsException:
                raise ValidationError(message="Invalid token.")

    token = prompts.secret(
        message="Enter a personal access token with appropriate permissions.",
        long_instruction="Need help? https://token.schemarin.celsiusnarhwal.dev",
        validate=TokenValidator(),
        transformer=lambda result: f"Signed in as {GitHub(result).get_user().login}"
    ).execute()

    return token


def resolve_permission_error(path: Path, exception: PermissionError, spinner: Halo):
    failed_path = Path(exception.filename)
    protected = [Path(x) for x in
                 (["/System", "/usr", "/bin", "/sbin", "/var"] +
                  [*toml.load(Path(__file__).parent / "protected.toml")["apps"].values()])]

    if any(failed_path.realpath().startswith(p.realpath()) for p in protected):
        if failed_path == path:
            msg = f"{path} is protected by macOS. "
        else:
            msg = f"{failed_path}, a subdirectory of {path}, is protected by macOS. "
        msg += (f"To allow Schemarin access to it, you must disable System Integrity Protection "
                f"{click.style('(very bad idea)', fg='red', bold=TabError)} and rerun Schemarin with sudo.")

        spinner.fail(msg)

        info = "Learn more: https://schemarin.celsiusnarhwal.dev/blob/HEAD/schemarin-and-sip.md"
        click.echo(make_info_text(info))
    else:
        macos_version = platform.mac_ver()[0].split(".")
        macos_version = ".".join(macos_version[:2])
        if failed_path == path:
            msg = f"You don't have permissions to access {path}. "
        else:
            msg = f"You don't have permission to access {failed_path}, which is a subdirectory of {path}. "

        msg += "Try running Schemarin with sudo."
        spinner.fail(msg)

        click.echo(make_info_text(f"Learn more: https://support.apple.com/guide/mac-help/"
                                  f"change-permissions-for-files-folders-or-disks-mchlp1203/{macos_version}"))



import plistlib
import re
import tempfile
from collections import deque

import click
import inflect as ifl
import pygit2
import validators
from InquirerPy.base import Choice
from github import Github as GitHub
from github.GithubException import UnknownObjectException
from halo import Halo
from path import Path
from prompt_toolkit.document import Document
from prompt_toolkit.validation import Validator, ValidationError

import helpers
import prompts

inflect = ifl.engine()


def add_from_builtins():
    add(helpers.get_builtin_schemes_dir())


def add_from_github():
    gh = GitHub()
    repo_regex = re.compile(r"^(?P<owner>[^/]+)/(?P<repo>[^/]+)$")

    def repo_name_processor(result: str) -> str:
        url_regex = re.compile(r"(?<=github.com/)[a-zA-Z0-9-_.]+/[a-zA-Z0-9-_.]+")
        match = url_regex.search(result, re.IGNORECASE)
        if match:
            return match.group(0)
        else:
            return result

    while True:
        token = None

        repo_name = prompts.text(
            message="Enter the name or URL of a GitHub repository.",
            instruction="e.g. celsiusnarhwal/beenine",
            validate=lambda result: validators.url(result) or repo_regex.match(result),
            filter=repo_name_processor,
            invalid_message="Invalid GitHub repository.",
        ).execute()

        try:
            repo = gh.get_repo(repo_name)
        except UnknownObjectException:
            msg = f"{repo_name} may be private. You'll need to sign in to GitHub."
            click.echo(helpers.make_info_text(msg))

            token = helpers.sign_in_to_github()

            if token is None:
                continue
            elif token is False:
                click.echo(helpers.make_error_text("Schemarin was denied access to your GitHub account."))
                continue
            else:
                gh = GitHub(token)
                try:
                    repo = gh.get_repo(repo_name)
                except UnknownObjectException:
                    msg = f"{repo_name} either doesn't exist or isn't accessible to your GitHub account."
                    click.echo(helpers.make_error_text(msg))
                    continue

        tmpdir = tempfile.mkdtemp()

        with Halo(text="Loading repository contents...", spinner="dots") as spinner:
            try:
                callbacks = pygit2.RemoteCallbacks(pygit2.UserPass(token, "x-oauth-basic"))
                pygit2.clone_repository(repo.clone_url, tmpdir, callbacks=callbacks)
            except pygit2.GitError:
                try:
                    callbacks = pygit2.RemoteCallbacks(pygit2.UserPass(gh.get_user().login, token))
                    pygit2.clone_repository(repo.clone_url, tmpdir, callbacks=callbacks)
                except pygit2.GitError:
                    isnt = click.style("isn't", underline=True)  # because f-strings can't have backslashes, lol
                    spinner.fail(f"There was an error cloning {repo.full_name}. This {isnt} your fault. "
                                 f"Try again later.")
                    continue
                else:
                    spinner.succeed("Repository loaded!")
                    add(Path(tmpdir))
                    break
            else:
                spinner.succeed("Repository loaded!")
                add(Path(tmpdir))
                break


def add_from_files():
    class FilepathValidator(Validator):
        def validate(self, document: Document) -> None:
            if not Path(document.text).exists():
                raise ValidationError(message="That path doesn't exist.")
            if Path(document.text).isfile() and Path(document.text).ext != ".itermcolors":
                raise ValidationError(message="That's not an iTermColors file.")

    path: Path = prompts.filepath(
        message="Enter the path to a iTermColors file or a directory containing some.",
        instruction="Relative paths and symbolic links are fully supported.",
        long_instruction=f"The current working directory is {Path.getcwd().realpath()}.",
        validate=FilepathValidator(),
        filter=lambda result: Path(result).expand().realpath(),
        transformer=lambda result: Path(result).expand().realpath(),
    ).execute()

    if path.isfile():
        with tempfile.TemporaryDirectory() as tmpdir:
            td_path = Path(tmpdir)
            path.copy(td_path)
            add(td_path)
    else:
        add(path)


def add(path: Path):
    with Halo(text="Looking for color schemes...", spinner="dots") as spinner:
        try:
            colorfiles = {p.realpath(): plistlib.load(p.open("rb")) for p in path.walk("*.itermcolors")}
        except PermissionError as e:
            helpers.resolve_permission_error(path, e, spinner)
            raise KeyboardInterrupt
        else:
            existing = helpers.get_scheme_settings()
            deque(maxlen=0).extend(
                colorfiles.pop(p.realpath(), None)
                for key in existing.keys() for p in path.walk(f"{key}.itermcolors")
            )

        if not colorfiles:
            if path == helpers.get_builtin_schemes_dir():
                spinner.fail("You've already added all the built-in color schemes. Impressive!")
            else:
                spinner.fail("There are no color schemes there that you don't already have.")
            raise KeyboardInterrupt
        elif len(colorfiles) == 1:
            schemes = [colorfiles][0]
            spinner.info("There's only one color scheme here that you don't already have, so Schemarin will go ahead "
                         "and add it.")
        else:
            spinner.succeed("Color schemes loaded!")
            schemes = prompts.fuzzy(
                message="Choose some color schemes.",
                instruction="Type to search. Color schemes you already have are not listed.",
                long_instruction=helpers.MULTISELECT_CONTROLS,
                choices=sorted(
                    [Choice(name=path.stem, value=path) for path in colorfiles],
                    key=lambda choice: choice.name),
                multiselect=True,
                transformer=lambda result: f"Adding {inflect.no('color scheme', len(result))}.",
            ).execute()

    scheme_settings = helpers.get_scheme_settings()

    for scheme in schemes:
        scheme_settings[scheme.stem] = plistlib.load(scheme.open("rb"))

    helpers.save_scheme_settings(scheme_settings)

    msg = f"Added {helpers.get_scheme_indicator(schemes)}. Restart iTerm2 to apply your changes."
    click.echo(helpers.make_success_text(msg))

    if path.realpath().startswith(Path(tempfile.gettempdir()).realpath()):
        path.rmtree()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['schemarin']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'colorama>=0.4.6,<0.5.0',
 'colour>=0.1.5,<0.2.0',
 'halo>=0.0.31,<0.0.32',
 'inflect>=6.0.2,<7.0.0',
 'inquirerpy>=0.3.4,<0.4.0',
 'loctocat>=1.0.2,<2.0.0',
 'path>=16.5.0,<17.0.0',
 'pygit2>=1.11.1,<2.0.0',
 'pygithub>=1.57,<2.0',
 'pyperclip>=1.8.2,<2.0.0',
 'toml>=0.10.2,<0.11.0',
 'validators>=0.20.0,<0.21.0']

entry_points = \
{'console_scripts': ['schemarin = schemarin.schemarin:cli']}

setup_kwargs = {
    'name': 'schemarin',
    'version': '1.0.1',
    'description': 'A color scheme manager for iTerm2',
    'long_description': "# Schemarin\n\nSchemarin is a color scheme manager for [iTerm2](https://iterm2.com) that should really be installed with \n[Homebrew](https://brew.sh) rather than pip.\n\n```bash\nbrew tap celsiusnarhwal/htt\nbrew install schemarin\n```\n\nSchemarin is published on PyPI **for technical reasons only** and is not intended to be installed with pip or other\nPython package managers.\n\nFor more information, [check out Schemarin's GitHub repository](https://schemarin.celsiusnarhwal.dev).",
    'author': 'celsius narhwal',
    'author_email': 'hello@celsiusnarhwal.dev',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://schemarin.celsiusnarhwal.dev',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
